@@ -24,7 +24,6 @@ from gettext import gettext as _
 
 from gi.repository import GLib
 from gi.repository import GObject
-from gi.repository import Pango
 from gi.repository import Gtk
 from gi.repository import Gdk
 
@@ -35,6 +34,8 @@ from sugar3.graphics.icon import Icon, CellRendererIcon
 from sugar3.graphics.xocolor import XoColor
 from sugar3.graphics.alert import Alert
 from sugar3.graphics.palettemenu import PaletteMenuItem
+from sugar3.graphics.scrollingdetector import ScrollingDetector
+from sugar3.graphics.palettewindow import TreeViewInvoker
 from sugar3.datastore import datastore
 
 from jarabe.model import bundleregistry
@@ -68,20 +69,15 @@ class ActivitiesTreeView(Gtk.TreeView):
         self._model.set_visible_func(self.__model_visible_cb)
         self.set_model(self._model)
 
-        column = Gtk.TreeViewColumn()
-
-        cell_favorites = []
         for i in range(desktop.get_number_of_views()):
-            cell_favorites.append(CellRendererFavorite(self, i))
-            cell_favorites[i].connect('clicked', self.__favorite_clicked_cb)
-            column.pack_start(cell_favorites[i], True)
-            column.set_cell_data_func(cell_favorites[i],
-                                      self.__favorite_set_data_cb)
+            column = Gtk.TreeViewColumn()
+            cell = CellRendererFavorite(i)
+            cell.connect('clicked', self.__favorite_clicked_cb)
+            column.pack_start(cell, True)
+            column.set_cell_data_func(cell, self.__favorite_set_data_cb)
+            self.append_column(column)
 
-        self.append_column(column)
-
-        cell_icon = CellRendererActivityIcon(self)
-        cell_icon.connect('erase-activated', self.__erase_activated_cb)
+        cell_icon = CellRendererActivityIcon()
         cell_icon.connect('clicked', self.__icon_clicked_cb)
 
         column = Gtk.TreeViewColumn()
@@ -135,9 +131,8 @@ class ActivitiesTreeView(Gtk.TreeView):
         self.set_search_column(self._model.column_title)
         self.set_enable_search(False)
         self._activity_selected = None
-
-    def __erase_activated_cb(self, cell_renderer, bundle_id):
-        self.emit('erase-activated', bundle_id)
+        self._invoker = TreeViewInvoker()
+        self._invoker.attach_treeview(self)
 
     def __favorite_set_data_cb(self, column, cell, model, tree_iter, data):
         favorite = \
@@ -187,6 +182,20 @@ class ActivitiesTreeView(Gtk.TreeView):
         if column == self._icon_column:
             self._start_activity(path)
 
+    def create_palette(self, path, column):
+        if column == self._icon_column:
+            row = self.get_model()[path]
+            bundle_id = row[self.get_model().column_bundle_id]
+
+            registry = bundleregistry.get_registry()
+            palette = ActivityListPalette(registry.get_bundle(bundle_id))
+            palette.connect('erase-activated', self.__erase_activated_cb,
+                            bundle_id)
+            return palette
+
+    def __erase_activated_cb(self, palette, event, bundle_id):
+        self.emit('erase-activated', bundle_id)
+
     def get_activities_selected(self):
         activities = []
         for row in self.get_model():
@@ -222,6 +231,16 @@ class ActivitiesTreeView(Gtk.TreeView):
     def __get_last_activity_error_handler_cb(self, entries, total_count):
         pass
 
+    def connect_to_scroller(self, scrolled):
+        scrolled.connect('scroll-start', self._scroll_start_cb)
+        scrolled.connect('scroll-end', self._scroll_end_cb)
+
+    def _scroll_start_cb(self, event):
+        self._invoker.detach()
+
+    def _scroll_end_cb(self, event):
+        self._invoker.attach_treeview(self)
+
 
 class ListModel(Gtk.TreeModelSort):
     __gtype_name__ = 'SugarListModel'
@@ -334,8 +353,8 @@ class ListModel(Gtk.TreeModelSort):
 class CellRendererFavorite(CellRendererIcon):
     __gtype_name__ = 'SugarCellRendererFavorite'
 
-    def __init__(self, tree_view, favorite_view):
-        CellRendererIcon.__init__(self, tree_view)
+    def __init__(self, favorite_view):
+        CellRendererIcon.__init__(self)
 
         self.favorite_view = favorite_view
         self.props.width = style.GRID_CELL_SIZE
@@ -343,9 +362,6 @@ class CellRendererFavorite(CellRendererIcon):
         self.props.size = style.SMALL_ICON_SIZE
         self.props.icon_name = desktop.get_favorite_icons()[favorite_view]
         self.props.mode = Gtk.CellRendererMode.ACTIVATABLE
-        prelit_color = profile.get_color()
-        self.props.prelit_stroke_color = prelit_color.get_stroke_color()
-        self.props.prelit_fill_color = prelit_color.get_fill_color()
 
 
 class CellRendererActivityIcon(CellRendererIcon):
@@ -356,8 +372,8 @@ class CellRendererActivityIcon(CellRendererIcon):
                             ([str])),
     }
 
-    def __init__(self, tree_view):
-        CellRendererIcon.__init__(self, tree_view)
+    def __init__(self):
+        CellRendererIcon.__init__(self)
 
         self.props.width = style.GRID_CELL_SIZE
         self.props.height = style.GRID_CELL_SIZE
@@ -370,21 +386,6 @@ class CellRendererActivityIcon(CellRendererIcon):
         self.props.prelit_stroke_color = prelit_color.get_stroke_color()
         self.props.prelit_fill_color = prelit_color.get_fill_color()
 
-        self._tree_view = tree_view
-
-    def create_palette(self):
-        model = self._tree_view.get_model()
-        row = model[self.props.palette_invoker.path]
-        bundle_id = row[model.column_bundle_id]
-
-        registry = bundleregistry.get_registry()
-        palette = ActivityListPalette(registry.get_bundle(bundle_id))
-        palette.connect('erase-activated', self.__erase_activated_cb)
-        return palette
-
-    def __erase_activated_cb(self, palette, bundle_id):
-        self.emit('erase-activated', bundle_id)
-
 
 class ClearMessageBox(Gtk.EventBox):
     def __init__(self, message, button_callback):
@@ -453,6 +454,8 @@ class ActivitiesList(Gtk.VBox):
         self._tree_view.connect('erase-activated', self.__erase_activated_cb)
         self._scrolled_window.add(self._tree_view)
         self._tree_view.show()
+        scrolling_detector = ScrollingDetector(self._scrolled_window)
+        self._tree_view.connect_to_scroller(scrolling_detector)
 
         self._alert = None
         self._clear_message_box = None
